//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Item/ParticleCoreShellItem.h
//! @brief     Defines class ParticleCoreShellItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_ITEM_PARTICLECORESHELLITEM_H
#define BORNAGAIN_GUI_MODEL_ITEM_PARTICLECORESHELLITEM_H

#include "GUI/Model/Item/ItemWithParticles.h"
#include <memory>

class ParticleCoreShell;
class ParticleItem;
class MaterialItems;

class ParticleCoreShellItem : public ItemWithParticles {
public:
    ParticleCoreShellItem(const MaterialItems* materials);
    void serialize(Serializer& s) override;

    std::unique_ptr<ParticleCoreShell> createParticleCoreShell() const;

    ParticleItem* core() const;
    void clearCore();
    ParticleItem* createCore(MaterialItems* materials);

    ParticleItem* shell() const;
    void clearShell();
    ParticleItem* createShell(MaterialItems* materials);

    QVector<ItemWithParticles*> containedItemsWithParticles() const override;

private:
    std::unique_ptr<ParticleItem> m_core;
    std::unique_ptr<ParticleItem> m_shell;
    const MaterialItems* m_materials;
};

#endif // BORNAGAIN_GUI_MODEL_ITEM_PARTICLECORESHELLITEM_H
