//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Item/ResolutionFunctionItems.h
//! @brief     Defines family of ResolutionFunctionItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2022
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_ITEM_RESOLUTIONFUNCTIONITEMS_H
#define BORNAGAIN_GUI_MODEL_ITEM_RESOLUTIONFUNCTIONITEMS_H

#include "GUI/Model/Types/DoubleProperty.h"
#include <memory>
#include <variant>

class IResolutionFunction2D;
class Serializer;

class ResolutionFunctionItem {
public:
    virtual ~ResolutionFunctionItem() = default;

    virtual std::unique_ptr<IResolutionFunction2D>
    createResolutionFunction(double scale = 1.0) const = 0;

    //! Set the unit of the distributed value
    void setUnit(const std::variant<QString, Unit>& unit) { m_unit = unit; }
    std::variant<QString, Unit> unit() const { return m_unit; }

    virtual void serialize(Serializer&){};

protected:
    std::variant<QString, Unit> m_unit;
};

class ResolutionFunctionNoneItem : public ResolutionFunctionItem {
public:
    std::unique_ptr<IResolutionFunction2D>
    createResolutionFunction(double scale = 1.0) const override;
};

class ResolutionFunction2DGaussianItem : public ResolutionFunctionItem {
public:
    ResolutionFunction2DGaussianItem();
    std::unique_ptr<IResolutionFunction2D>
    createResolutionFunction(double scale = 1.0) const override;

    void serialize(Serializer& s) override;

    DOUBLE_PROPERTY(sigmaX, SigmaX);
    DOUBLE_PROPERTY(sigmaY, SigmaY);
};

#endif // BORNAGAIN_GUI_MODEL_ITEM_RESOLUTIONFUNCTIONITEMS_H
