//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/VectorEditor.cpp
//! @brief     Implements class VectorEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Instrument/VectorEditor.h"
#include <QDoubleSpinBox>
#include <QFormLayout>
#include <QGroupBox>
#include <limits>

VectorEditor::VectorEditor(const QString& title, const QString& tooltip, QWidget* parent)
    : QGroupBox(title, parent)
{
    auto* form = new QFormLayout(this);

    m_xSpinBox = new QDoubleSpinBox(this);
    m_xSpinBox->setToolTip(tooltip.arg("x coordinate"));
    m_xSpinBox->setDecimals(4);
    m_xSpinBox->setRange(std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max());
    form->addRow("X:", m_xSpinBox);

    m_ySpinBox = new QDoubleSpinBox(this);
    m_ySpinBox->setToolTip(tooltip.arg("y coordinate"));
    m_ySpinBox->setDecimals(4);
    m_ySpinBox->setRange(std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max());
    form->addRow("Y:", m_ySpinBox);

    m_zSpinBox = new QDoubleSpinBox(this);
    m_zSpinBox->setToolTip(tooltip.arg("z coordinate"));
    m_zSpinBox->setDecimals(4);
    m_zSpinBox->setRange(std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max());
    form->addRow("Z:", m_zSpinBox);

    connect(m_xSpinBox, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &VectorEditor::onXValueChanged);
    connect(m_ySpinBox, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &VectorEditor::onYValueChanged);
    connect(m_zSpinBox, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &VectorEditor::onZValueChanged);
}

void VectorEditor::setVector(const VectorDescriptor& d)
{
    m_descriptor = d;
    m_xSpinBox->setValue(m_descriptor.x);
    m_ySpinBox->setValue(m_descriptor.y);
    m_zSpinBox->setValue(m_descriptor.z);
}

void VectorEditor::onXValueChanged(double x)
{
    if (x != m_descriptor.x) {
        m_descriptor.x.set(x);
        emit vectorChanged();
    }
}

void VectorEditor::onYValueChanged(double y)
{
    if (y != m_descriptor.y) {
        m_descriptor.y.set(y);
        emit vectorChanged();
    }
}

void VectorEditor::onZValueChanged(double z)
{
    if (z != m_descriptor.z) {
        m_descriptor.z.set(z);
        emit vectorChanged();
    }
}
